%% Init fil

%% Create buses 
%path Signals
Simulink.Bus.createObject('path_bus','path_bus/BusCreator');
Simulink.Bus.createObject('path_signals_bus','path_signals_bus/BusCreator');
Simulink.Bus.createObject('ref_signals_bus','ref_signals_bus/BusCreator');

%gains, hydrodyn, etc
Simulink.Bus.createObject('init_constants_bus','init_constants_bus/BusCreator');
%Simulink.Bus.createObject('test_backstep_bus','test_backstep_bus/BusCreator');

% Dynamic limits 
Simulink.Bus.createObject('dyn_lim_bus','dyn_lim_bus/BusCreator');

%State Machine
Simulink.Bus.createObject('path_properties_bus','path_properties_bus/BusCreator'); %Path propertie
Simulink.Bus.createObject('tasks_bus','tasks_bus/BusCreator');                     % Attributes about tasks
Simulink.Bus.createObject('complete_task_bus','complete_task_bus/BusCreator');     % Completed/not completed tasks

Simulink.Bus.createObject('hazard_bus', 'hazard_bus/BusCreator');




%%
nu0 = [0 0 0 0 0 0];
eta0 = [0 0 0 0 0 0];

%% Mission init
init_mission

%% Rigid body inertia system
r_g = [0 0 0]'; %location of CG with respect to CO
m = 20; % mass
lambda = 0.99 ;
%inertia components
ixx= 0.8094;
ixy= -0.00308;
ixz=-0.01546;
iyy=0.87794;
iyz= 0.06283; 
izz= 1.57147;


%Inertia matrix
I_g = [ixx -ixy -ixz;
       -ixy iyy -iyz;
       -ixz -iyz izz];
   
% Rigid body inertia system matrix
M_RB = [m*eye(3) -m*Smtrx(r_g)
        m*Smtrx(r_g) I_g];
    
%% Linear hydrodynamic matrices
%Linear damping matrix

D = diag([5 5 3 1 1 2]); %10 10 30 5 5 1
Unmodeled_D = [ -1 -1 0 0 0 1;
                -1 -3 0 0 0 3;
                0 0 0 0 0 0;
                0 0 0 0 0 0;
                0 0 0 0 0 0;
                1 1 0 0 0 -1];
%Unmodeled_D = zeros(6,6);

%Added Mass matrix, Zero frequency M_A(0) 
M_A = [10.7182 0 0 0 0 0 
        0 10.7182 0 0 0 0
        0 0 40.468 0 0 0
        0 0 0 5.2481 0 0
        0 0 0 0 5.2481 0
        0 0 0 0 0 2.4006];
              
    

% Total mass matrix
M = M_RB + M_A;

Unmodeled_M = [-4 -5 0 0 0 1;
                -3 -3 0 0 0 3;
                0 0 0 0 0 0;
                0 0 0 0 0 0;
                0 0 0 0 0 0;
                1 3 0 0 0 2];

%Unmodeled_D = zeros(6,6);
%Inverted Mass matrix
M_inv = inv(M);

M_3x3 = [M(1,1) M(1,2) M(1,6);
        M(2,1) M(2,2) M(2,6);
        M(6,1) M(6,2) M(6,6)];
    
M_controller = [M(1,1) 0 0;
                0   M(2,2) 0;
                0   0   M(6,6)];

D_3x3 = [D(1,1) D(1,2) D(1,6);
        D(2,1) D(2,2) D(2,6);
        D(6,1) D(6,2) D(6,6)];
    
D_controller = [D(1,1) 0 0;
                0   D(2,2) 0;
                0   0   M(6,6)];
    
M = M + Unmodeled_M;
D = D + Unmodeled_D;
    
    
init_const = struct('M',zeros(3,3),'D',zeros(3,3),'kappa_1',zeros(1,1),'K_1',zeros(3,3),'K_2',zeros(3,3), 'K_i', zeros(3,3), ...
    'M_controller',zeros(3,3),'D_controller', zeros(3,3));
init_const.M = M_3x3;
init_const.D = D_3x3;
init_const.kappa_1 = 0;
init_const.K_1 = [2 0 0; 0 2 0; 0 0 1]*0.05;%0.1;
init_const.K_2 =  [8 0 0; 0 8 0; 0 0 2]*7.5;%*12.45;
init_const.K_i = [1 0 0; 0 1 0; 0 0 1]*0; %17
init_const.M_controller = M_controller;
init_const.D_controller = D_controller;
clear ixx ixy ixz iyy iyz izz m M_A M_RB r_g I_g




%% Dynamic constraints
dyn_lim = struct('vel_limit', 0, 'accel_limit', 0);
dyn_lim.vel_limit = 0.1;
dyn_lim.accel_limit = 0.08;
%% Run Simulation
model='sim_model';
load_system(model);
tstop=650;
%Running the simulation
tic;
%warning off
 simOut=sim(model,'SaveOutput','on',...
        'SaveFormat','StructureWithTime',...
        'ReturnWorkspaceOutputs','on',...
        'SaveOutput','on','OutputSaveName','res_gun',...
        'StartTime','0','StopTime',num2str(tstop) );
toc
    
%% Extracting Values from Simulation
%eta_d = simOut.logsout.find("eta_d").Values.Data;
eta_d   = simOut.logsout.find('RefSignals').Values.eta_d.Data;
eta_d_s = simOut.logsout.find('RefSignals').Values.eta_d_s.Data;
eta     = simOut.logsout.find('eta').Values.Data;
u_d     = simOut.logsout.find('u_d').Values.Data;
u_d_dot     = simOut.logsout.find('u_d_dot').Values.Data;
s       = simOut.logsout.find('s').Values.Data;

%u = simOut.logsout.find("eta").Values.Data;
tau = simOut.logsout.find('tau').Values.Data;
Time = simOut.logsout.find('eta').Values.Time;
%err = simOut.logsout.find('error').Values.Data;
%clf(figure(2))
%plot(Time, u, Time, u_d);

figure(2); hold on;
pd_d_plot  = plot(eta_d(:,2),eta_d(:,1),'b','LineWidth',1.25); M1 = "eta_d";
p_plot    = plot(eta(:,2),eta(:,1),'r','LineWidth',1.25);     M2 = "eta";

legend([pd_d_plot,p_plot],[M1, M2]);
hold off; xlabel('y-position'); ylabel('x-position');
% 
figure(3); hold on;
yaw_d_plot = plot(Time,eta_d(:,3)); M3 = "yaw_d";
yaw_plot   = plot(Time, eta(:,3));  M4  = "yaw";
legend([yaw_d_plot,yaw_plot],[M3, M4]);
hold off; xlabel('time'); ylabel('yaw angle');




figure(4); hold on;
tau_1_plot = plot(Time,tau(:,1)); T1 = "tau_{surge}";
legend(tau_1_plot, T1);
hold off; xlabel('time'); ylabel('Force [N]')

figure(5); hold on;
tau_2_plot = plot(Time,tau(:,2)); T2 = "tau_{sway}";
legend(tau_2_plot, T2);
hold off; xlabel('time'); ylabel('Force [N]')

% figure(6); hold on;
% tau_3_plot = plot(Time,tau(:,6)); T3 = "tau_{yaw}";
% legend(tau_3_plot, T3);
% hold off; xlabel('time'); ylabel('Moment  [Nm]')

 figure(7); hold on;
 u_d_plot = plot(Time, u_d); u_d_pl_leg = "u_d";
 legend(u_d_plot,u_d_pl_leg);
 hold off; xlabel('Time [s]'); ylabel('speed [m/s]');
% x_d_s_plot = plot(Time, eta_d_s(:,1)); dsP_x = "x_d^s";
% y_d_s_plot = plot(Time, eta_d_s(:,2)); dsP_y = "x_d^s";
% legend([x_d_s_plot, y_d_s_plot],[dsP_x, dsP_y]);
% hold off;


figure(8); hold on;
x_time_plot = plot(Time,eta(:,1)); xx = "x";
x_d_time_plot = plot(Time,eta_d(:,1)); xd = "x_d";
legend([x_time_plot, x_d_time_plot],[xx, xd]);

hold off; xlabel('Time [sec]'); ylabel('position [m]');


figure(9); hold on;
y_time_plot = plot(Time,eta(:,2),'b','LineWidth',1.25); yy = "y";
y_d_time_plot = plot(Time,eta_d(:,2),'r','LineWidth',1.25); yd = "y_d";
legend([y_time_plot, y_d_time_plot],[yy, yd]);
hold off; xlabel('Time [sec]'); ylabel('position [m]');


xi = simOut.logsout.find('xi').Values.Data;
z_2 = simOut.logsout.find('z2').Values.Data;

K_i = init_const.K_i;

figure(10); hold on;
xi_plot_x = plot(Time,K_i(1,1)*xi(:,1)); xi_label_1 = "\Gamma\xi(1)";
z2_plot_x = plot(Time,z_2(:,1)); z2_label_1 ="z2(1)";
legend([xi_plot_x, z2_plot_x], [xi_label_1, z2_label_1]);

hold off; xlabel('time [s]'); ylabel('force [N]');

figure(11); hold on;
ud_plot = plot(s,u_d);
hold off; xlabel('s [-]'); ylabel('speed [m/s]');
